/*
 *  Copyright IBM Corp. 2010
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at:
 * 
 * http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * permissions and limitations under the License.
 */
package org.openntf.attachmentviewer;

import java.io.File;
import org.apache.commons.codec.binary.Base64;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.browser.Browser;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IPartListener2;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchPartReference;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;
import org.openntf.attachmentviewer.services.IAttachmentViewerProvider;
import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;
import org.osgi.framework.BundleException;
import org.osgi.framework.InvalidSyntaxException;
import org.osgi.framework.ServiceReference;

import com.ibm.rcp.ui.IAttachment;

public class Viewer extends ViewPart {

	Browser browser = null;
	
	protected static final String WEB_CONTAINER = "com.ibm.pvc.webcontainer";
	
	class SpecialListener implements ISelectionListener{
		private Viewer viewer = null;
		
		public SpecialListener(Viewer viewer){
			this.viewer = viewer;
		}
		
		public void selectionChanged(IWorkbenchPart arg0, ISelection arg1) {
			
			if (viewer.getSite().getPage().isPartVisible(viewer) == false)
				return;
									
			if (arg1 instanceof IStructuredSelection){
				final IStructuredSelection ss = (IStructuredSelection)arg1;
				
				Job processSels = new Job("Process Attachments"){

					protected IStatus run(IProgressMonitor arg0) {

						setDebug();
						
						processSelections(ss.toArray());
						
						return Status.OK_STATUS;
					}					
				};
					
				processSels.schedule();
			}
		}		
	}
	SpecialListener listener = null;
	
	public void createPartControl(Composite parent) {
		try {
			listener = new SpecialListener(this);
			
			Bundle bundle = Platform.getBundle( WEB_CONTAINER );
			bundle.start();
		
			//browser = new WebBrowser(parent,SWT.NONE, WebBrowser.XULRUNNER_SERVICE);
			browser = new Browser(parent, SWT.NONE);
			
			int port = Activator.getDefault().getPortNumber();
			String host = Activator.getDefault().getHostName();
			
			initURL(port, host);
	
			IWorkbenchWindow[] windows = PlatformUI.getWorkbench().getWorkbenchWindows();
			
			for (int x=0; x<windows.length; x++){
				windows[x].getSelectionService().addSelectionListener(listener);
			}
						
		} catch (BundleException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}		
	}

	private void setDebug(){
		final IPreferenceStore prefs = Activator.getDefault().getPreferenceStore();
		
		final boolean debug = prefs.getBoolean("debug");
		
		Display d = Display.getDefault();
		
		
		if (d != null){
			d.syncExec(new Runnable(){
	
				public void run() {
					String cmd = "setDebug(" + (debug ? "true":"false" ) + ");";
					if (browser.execute(cmd) == false){
						
						//System.out.println(browser.getDocument().getXmlVersion());
					}
				}
			});
		}				

		
	}
	private void initURL(int port, String host) {
		browser.setUrl("http://" + host + ":" + port + "/atviewer/viewer.html");
	}
	
	public void dispose() {
		IWorkbenchWindow[] windows = PlatformUI.getWorkbench().getWorkbenchWindows();
		
		for (int x=0; x<windows.length; x++){
			windows[x].getSelectionService().removeSelectionListener(listener);
		}
		
		super.dispose();
	}

	public static String getPreviewURL(String fileName){
//		int port = Activator.getDefault().getPortNumber();
//		String host = Activator.getDefault().getHostName();
		String extension = fileName.substring(fileName.lastIndexOf(".")+1, fileName.length());
		String bPath = new String(Base64.encodeBase64(fileName.getBytes()));
//		String baseURL = "http://" + host + ":" + port;
		
		return "/proxy/" + bPath + "." + extension;
	}

	protected void processSelections(Object[] array) {
		StringBuffer jsonObj = new StringBuffer("[");
		int port = Activator.getDefault().getPortNumber();
		String host = Activator.getDefault().getHostName();
		String baseURL = "http://" + host + ":" + port;
				
		for (int x=0; x<array.length; x++){
			if (array[x] instanceof IAttachment){
				IAttachment ats = (IAttachment)array[x];
				File f = ats.getTempFile();
				
				if (f == null) break;

				String name = ats.getName();
				String extension = name.substring(name.lastIndexOf(".")+1, name.length()).toLowerCase();
				String surl = getPreviewURL(f.getAbsoluteFile().toString());
				String bPath = new String(Base64.encodeBase64(f.getAbsolutePath().getBytes()));
				
				BundleContext ctx = Activator.getBundleContext();
				
				ServiceReference[] refs = null;

				try {
					refs = ctx.getServiceReferences(IAttachmentViewerProvider.class.getName(), "(type=*." + extension + "*)");
				} catch (InvalidSyntaxException e) {
				}
				
				String content_url = baseURL + "/proxy/content/" + bPath + "." + extension;
				String script_url = baseURL + "/proxy/script/" + bPath + "." + extension;
				
				//If there is a provider then pass that along
				boolean hasProvider = (refs != null && refs.length > 0);
				boolean hasScript = false;
				
				if (refs != null && refs.length > 0){
					IAttachmentViewerProvider provider = (IAttachmentViewerProvider)ctx.getService(refs[0]);
					
					hasScript = provider.hasScript();
					
					ctx.ungetService(refs[0]);
				}
				
				jsonObj.append("{name: '" + name + "'" +
						",preview_url: '" + surl + "'" +
						",url: '" + f.toURI().toASCIIString() + "'" +
						",content_url:'" + (hasProvider ? content_url : "") + "'" +
						",script_url:'" + (hasProvider && hasScript ? script_url : "") + "'" +
						",type: '" + extension.toLowerCase() + "'");
				
				jsonObj.append("}");
				
				if (x < array.length-1){
					jsonObj.append(",");
				}
			}
		}
	
		jsonObj.append("]");
		
		Display d = Display.getDefault();
		
		final String jObj = jsonObj.toString();
		
		if (d != null){
			d.syncExec(new Runnable(){
	
				public void run() {
					String cmd = "showAttachments(" + jObj.toString() + ");";
					if (browser.execute(cmd) == false){
						
						//System.out.println(browser.getDocument().getXmlVersion());
					}
				}
			});
		}				
	}

	public void setFocus() {
		// TODO Auto-generated method stub
		
	}
	
}
