/*
 *  Copyright IBM Corp. 2010
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at:
 * 
 * http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * permissions and limitations under the License.
 */
package org.openntf.attachmentviewer.services;

import java.io.File;
import java.io.OutputStream;

/**
 * This is the base interface for the OSGI extension.  You should
 * extend AbstractFileProvider versus implementing this directly.
 * Most of the methods have been implemented in the abstract class.
 * 
 * Example registration of the service:
 * 
 * context.registerService(IAttachmentViewerProvider.class.getName(), new TextFileProvider(), properties);
 * 
 * @author Bob Balfe
 *
 */
public interface IAttachmentViewerProvider {

	
	/**
	 * Returns the HTTP content type this extension writes to.
	 * example:  image/jpeg
	 * 
	 * @return the content type in the form of a string
	 */
	public String getContentType();
	
	/**
	 * Identifies that this extension will provide JavaScript to extensions.js
	 * 
	 * @return true or false
	 */
	public boolean hasScript();
	
	/**
	 * Optional.
	 * 
	 * Write the actual file to the output stream.  Can be overridden for
	 * any type of conversion.  There is no need to close the OutputStream
	 * before exiting.
	 * 
	 * @param f - the local file (on the server) that will be read from
	 * @param os - the servlet outputstream to write to
	 */
	public void writeFile(File f, OutputStream os);
	
	
	/**
	 * Optional.
	 * 
	 * Write any HTML to the output stream.  This HTML will be inserted
	 * into a specific area where the attachment will be placed.  Some
	 * extensions just write a formatted HTML to the viewer place, this
	 * method would be used for that.
	 * 
	 * @param f - the local file (on the server) that will be read from
	 * @param os - the servlet outputstream to write to
	 * @param limit - the max limit of bytes the provider can write out,
	 * zero is unlimited.
	 */
	public void writeHTML(File f, OutputStream os, int limit);
	
	/**
	 * This method should return a JavaScript method with the signature:
	 * 
	 * function (id, url)
	 * 
	 * The id is the DOM element id where your code will be place.  The
	 * URL is the URL to the preview URL of the content.
	 * 
	 * @return the JavaScript as String
	 */
	public String getScript();
	
}
