/*
 *  Copyright IBM Corp. 2010
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at:
 * 
 * http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * permissions and limitations under the License.
 */
package org.openntf.attachmentviewer.services;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Enumeration;
import java.util.zip.ZipEntry;
import java.util.zip.ZipException;
import java.util.zip.ZipFile;

import org.openntf.attachmentviewer.Activator;
import org.openntf.attachmentviewer.Viewer;
import org.osgi.framework.BundleContext;
import org.osgi.framework.InvalidSyntaxException;
import org.osgi.framework.ServiceReference;
public class ZipFileProvider extends AbstractFileProvider {

	public String getContentType() {
		return "archive/zip";
	}

	@Override
	public void writeHTML(File f, OutputStream os, int limit) {
		try {
			ZipFile zf = new ZipFile(f);
			int idx = 0;
			BundleContext ctx = Activator.getBundleContext();
			ServiceReference[] refs = null;
			
			System.out.println("processing file: " + f.getAbsolutePath());
			
			//Dump the list of files with the anchors
			Enumeration<? extends ZipEntry> anchors = zf.entries();
			while (anchors.hasMoreElements()){
				ZipEntry entry = (ZipEntry)anchors.nextElement();
				String extension = entry.getName().substring(entry.getName().lastIndexOf(".")+1, entry.getName().length()).toLowerCase();
				idx++;

				try {
					refs = ctx.getServiceReferences(IAttachmentViewerProvider.class.getName(), "(type=*." + extension + "*)");
				} catch (InvalidSyntaxException e) {
				}

				if (refs == null || refs.length == 0){
					os.write( (idx + " - " + entry.getName() + "<br>\n").getBytes());
				}else{
					os.write( (idx + " - <a href=\"#" + entry.getName() + "\">" + entry.getName() + "</a><br>\n").getBytes());
				}
			}
			
			os.write("<hr>".getBytes());
			
			Enumeration<? extends ZipEntry> entries = zf.entries();
			idx = 0;
			
			while (entries.hasMoreElements()){
				ByteArrayOutputStream innerBuffer = new ByteArrayOutputStream();

				idx++;
				
				ZipEntry entry = (ZipEntry)entries.nextElement();
				String extension = entry.getName().substring(entry.getName().lastIndexOf(".")+1, entry.getName().length()).toLowerCase();
				
				try {
					refs = ctx.getServiceReferences(IAttachmentViewerProvider.class.getName(), "(type=*." + extension + "*)");
				} catch (InvalidSyntaxException e) {
				}
				
				if (refs == null || refs.length == 0) continue;
				
				System.out.println("processing file: " + entry.getName());

				String buf = "<a name=\"" + entry.getName() + "\"><b>File:</b> <i>" + entry.getName() + "</i></b></a><br><hr><div id=\"zip_" + idx + "\">";
				innerBuffer.write(buf.getBytes());

				IAttachmentViewerProvider avp = (IAttachmentViewerProvider)ctx.getService(refs[0]);
				
				//Copy the file to a local temporary file
				InputStream is = zf.getInputStream(entry);
				File out = File.createTempFile("zip-" + System.currentTimeMillis(), "." + extension);
				FileOutputStream fos = new FileOutputStream(out);
				
				byte[] bytes = new byte[1024];
				int bytesRead = 0;
				
				while ((bytesRead = is.read(bytes)) > 0){
					fos.write(bytes,0, bytesRead);
				}
				fos.close();
								
				ByteArrayOutputStream controlledBuffer = new ByteArrayOutputStream();
				avp.writeHTML(out, controlledBuffer, 20000);	
				
				innerBuffer.write(controlledBuffer.toByteArray());
				
				if (avp.hasScript()){
					String url = Viewer.getPreviewURL(out.getAbsolutePath());
					innerBuffer.write( ("<div id=\"zips_" + idx + "\"class=\"zip_script\" ext=\"." + extension + "\" url=\"" + url + "\"></div>").getBytes());
				}
				
				innerBuffer.write("</div><hr>".getBytes());
								
				os.write(innerBuffer.toByteArray());

				out.deleteOnExit();
				
				ctx.ungetService(refs[0]);
			}
			
		} catch (ZipException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	@Override
	public String getScript() {
		StringBuffer buf = new StringBuffer("function (id, url){\n");
		
		// Our script looks for all elements where the "zip_script" class is used
		// We process that block and than call the appropriate handlers script
		// with the id
		
		buf.append("\n\tdojo.query(\".zip_script\").forEach(function(node, index, arr){\n");

		buf.append("\n\tvar id = node.getAttribute(\"id\");");
		buf.append("\n\tvar ext = node.getAttribute(\"ext\");");
		buf.append("\n\tvar url = node.getAttribute(\"url\");");
		
    	buf.append("\n\n\tvar func = getScript(ext);\n\tif (func != null)\n\t\tfunc(id, url);");

		buf.append("\n\t\t});\n");

		// end of main function
		buf.append("\t}\n");
		
		return buf.toString();
	}

	@Override
	public boolean hasScript() {
		return true;
	}

	
}
